SystemOrganization addCategory: #TusLibros!

TestCase subclass: #TusLibrosTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!TusLibrosTest methodsFor: 'Cart Instances' stamp: 'JuanManuelTastzian 5/29/2013 18:23'!
emptyCart
	^Cart newEmptyCart! !

!TusLibrosTest methodsFor: '1 - Cart Test' stamp: 'JuanManuelTastzian 5/29/2013 20:01'!
test1CartShouldBeEmptyAfterCreation
	self assert: self emptyCart isEmpty! !

!TusLibrosTest methodsFor: '1 - Cart Test' stamp: 'JuanManuelTastzian 5/29/2013 20:57'!
test2CartSizeShouldBeGreaterThan0AfterAddingABookToIt
	| cart book catalog |
	
	book := 'aBook'.
	catalog := Bag new.
	catalog add: book.
	cart := Cart withCatalog: catalog.
	
	cart add: book.
	
	self assert: (cart size > 0).! !

!TusLibrosTest methodsFor: '1 - Cart Test' stamp: 'JuanManuelTastzian 5/29/2013 20:59'!
test3aBookShouldBeListedAfterBeingAddedToACart
	| cart book catalog |
	
	book := 'aBook'.
	catalog := Bag new.
	catalog add: book.
	cart := Cart withCatalog: catalog.
	
	cart add: book.
	self assert: (cart includes: book)! !

!TusLibrosTest methodsFor: '1 - Cart Test' stamp: 'JuanManuelTastzian 5/29/2013 20:59'!
test4aCartHasOnlyOneBookAfterAddingABookToAnEmptyCart
	| cart book catalog |
	
	book := 'aBook'.
	catalog := Bag new.
	catalog add: book.
	cart := Cart withCatalog: catalog.
	
	cart add: book.
	self assert: (cart includes: book).
	self assert: cart size = 1! !

!TusLibrosTest methodsFor: '1 - Cart Test' stamp: 'JuanManuelTastzian 5/29/2013 21:02'!
test5aBookCanBeAddedMoreThanOnce
	| cart book anotherBook catalog |
	
	book := 'aBook'.
	anotherBook := 'anotherBook'.
	catalog := Bag new.
	catalog add: book.
	catalog add: anotherBook.
	
	cart := Cart withCatalog: catalog.
	
	cart add: book.
	cart add: book.
	cart add: anotherBook.
	
	self assert: (cart includes: book).
	self assert: cart size = 3.
	self assert: (cart numberOfUnitsAddedOf: book) = 2.
	self assert: (cart numberOfUnitsAddedOf: anotherBook) = 1.! !

!TusLibrosTest methodsFor: '1 - Cart Test' stamp: 'JuanManuelTastzian 5/29/2013 20:52'!
test6booksThatAreNotOnTheCatalogCantBeAddedToCart
	| cart book |
	
	cart := Cart newEmptyCart.
	book := 'aBook'.
	
	self should: [cart add: book ]
		raise: Error 
		withExceptionDo: [ :anError | 
			self assert: anError description = Cart productNotInCatalog ]! !

!TusLibrosTest methodsFor: '1 - Cart Test' stamp: 'JuanManuelTastzian 5/29/2013 20:39'!
test7onlyBooksFromCatalogCanBeAddedToCart
	| cart book catalog |
	
	book := 'aBook'.
	catalog := Bag new.
	catalog add: book.
	cart := Cart withCatalog: catalog.
	
	cart add: book.
	
	self assert: (cart includes: book).
	self assert: cart size = 1.! !

Object subclass: #Cart
	instanceVariableNames: 'size books catalog'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

Cart class
	instanceVariableNames: 'size'!

Cart class
	instanceVariableNames: 'size'!

!Cart class methodsFor: 'class initialization' stamp: 'JuanManuelTastzian 5/29/2013 18:30'!
newEmptyCart
	^self new withoutBooks! !

!Cart class methodsFor: 'error handling' stamp: 'JuanManuelTastzian 5/29/2013 20:54'!
productNotInCatalog
	^ 'Error: The requested product is not present in the catalog'.! !

!Cart class methodsFor: 'class initialization' stamp: 'JuanManuelTastzian 5/29/2013 20:41'!
withCatalog: aCatalog 
	^ self new catalog: aCatalog.! !

!Cart methodsFor: 'accessing' stamp: 'JuanManuelTastzian 5/29/2013 20:53'!
add: aBook 
	(catalog includes: aBook) ifTrue: [books add: aBook] ifFalse: [ Error signal: 'The requested product is not present in the catalog' ]
	! !

!Cart methodsFor: 'accessing' stamp: 'JuanManuelTastzian 5/29/2013 18:36'!
books
	^ books! !

!Cart methodsFor: 'accessing' stamp: 'JuanManuelTastzian 5/29/2013 20:43'!
catalog: aCatalog 
	books := Bag new.
	catalog := aCatalog.! !

!Cart methodsFor: 'accessing' stamp: 'JuanManuelTastzian 5/29/2013 19:16'!
includes: aBook 
	^books includes: aBook! !

!Cart methodsFor: 'accessing' stamp: 'JuanManuelTastzian 5/29/2013 20:04'!
isEmpty
	^ self size = 0! !

!Cart methodsFor: 'accessing' stamp: 'JuanManuelTastzian 5/29/2013 19:26'!
numberOfUnitsAddedOf: aBook 
	^(books occurrencesOf: aBook)! !

!Cart methodsFor: 'as yet unclassified' stamp: 'JuanManuelTastzian 5/29/2013 18:38'!
size
	^books size! !

!Cart methodsFor: 'accessing' stamp: 'JuanManuelTastzian 5/29/2013 20:27'!
withoutBooks
	books := Bag new.
	catalog := Bag new.! !
